/**
 * OW2 Util
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SshShell.java 5639 2010-10-15 11:39:54Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.util.protocol.ssh.impl.jsch;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;

import org.ow2.util.file.FileUtils;
import org.ow2.util.protocol.api.IShell;
import org.ow2.util.protocol.api.IShellFormatter;
import org.ow2.util.protocol.api.ProtocolException;
import org.ow2.util.protocol.impl.shell.ShShellFormatter;
import org.ow2.util.protocol.ssh.api.SshProtocolConfig;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.JSchException;

/**
 *
 * @author Loris Bouzonnet
 */
public class SshShell implements IShell {

	private final SshProtocolSession protocolSession;

	private Channel channel;

	private PrintWriter printWriter;

	private final IShellFormatter formatter;

	public SshShell(final SshProtocolSession protocolSession, final IShellFormatter formatter) throws ProtocolException {
		this.protocolSession = protocolSession;
		this.formatter = formatter;
		init();
	}

	private void init()  throws ProtocolException {

		channel =
			protocolSession.createJschChannel("shell", null);
		channel.setInputStream(protocolSession.getConfig().getInputStream());
		channel.setOutputStream(protocolSession.getConfig().getOutputStream());
		try {
			channel.connect();
		} catch (final JSchException e) {
			throw new ProtocolException(e);
		}
		try {
			printWriter = new PrintWriter(channel.getOutputStream());
		} catch (final IOException e) {
			throw new ProtocolException(e);
		}
	}

	public void close() {
		FileUtils.close(printWriter);
		channel.disconnect();
	}

	public void execute(final String command) throws ProtocolException {
		printWriter.println(command);
		printWriter.flush();
	}

	public void setVariable(final String name, final String value) throws ProtocolException {
		execute(formatter.formatSet(name, value));
	}

	public void unsetVariable(final String name) throws ProtocolException {
		execute(formatter.formatUnset(name));
	}

	public static void main(final String[] arg) throws ProtocolException {


		final HashMap<String, String> context = new HashMap<String, String>();
		context.put(SshProtocolConfigFactory.PRIVATE_KEY_URL,
				System.getProperty("user.home") + File.separator + ".ssh" + File.separator + "id_rsa");
		context.put(SshProtocolConfigFactory.KNOWN_HOSTS,
				System.getProperty("user.home") + File.separator + ".ssh" + File.separator + "known_hosts");
		context.put(SshProtocolConfigFactory.HOST, arg[0]);
		context.put(SshProtocolConfigFactory.USER, arg[1]);

		final SshProtocolConfig config =
			SshProtocolConfigFactory.createSshProtocolConfig(null, context);
		config.setInputStream(System.in);
		config.setOutputStream(System.out);

		final SshProtocolSession sshSession =
			new SshProtocol().createSession(config);
		final IShellFormatter formatter = new ShShellFormatter();
		final SshShell shell = new SshShell(sshSession, formatter);
		shell.setVariable("welcome", "\"Hello " + System.getProperty("user.name") + "\"");
		shell.execute("echo " + formatter.formatVariable("welcome"));
	}

}
